include "Vram.inc"
include "VVram.inc"
include "Chars.inc"

ext MonoPattern, ColorPattern, SpritePattern, LinePattern
ext VVram_

CharPatternSize equ VramStep*CharHeight
SpritePatternSize equ 4*2*16
Top equ VramRowSize*1

dseg
SpriteChar:
    defb 0,0
SpriteLayer:
	defs VVramWidth*(VVramHeight+2)
Backup:
	defs VVramWidth*(VVramHeight+2)
CharRam: public CharRam
    defs CharPatternSize*Char_End

cseg
ColorSource:
defb 64,3 ; Ascii
defb 16,1 ; Logo
defb 2,2 ; Wall
defb 1,3 ; HardWall
defb 2,3 ; Meter
defb 2,2 ; Fence
defb 0
InitVram: public InitVram
    mov ax,0004h | int 10h ; Set video mode
    mov ax,0500h | int 10h ; Select active display page

    mov si,MonoPattern
    mov di,CharRam
    mov bx,ColorSource
    do
        mov ch,[bx]
        or ch,ch
    while nz
        inc bx
        mov dl,[bx] | inc bx
        call MakePatternMono
    wend

    mov si,ColorPattern
    mov ch,Char_End-Char_MyFort
    call MakePatternColor
ret

MakePatternMono:
;   di: destination
;   si: source
;   dl: color
;   ch: count
    mov dh,dl
    shr dh,1 | shr dh,1 | shr dh,1 | shr dh,1
    do
        mov cl,CharHeight
        do
            push cx
                mov al,[si] | inc si
                mov ch,VramStep
                do
                    xor ah,ah
                    mov cl,8/VramStep
                    do
                        shl ah,1 | shl ah,1
                        shl al,1
                        if c
                            or ah,dl
                        else
                            or ah,dh
                        endif
                        dec cl
                    while nz | wend
                    mov [di],ah | inc di
                    dec ch
                while nz | wend
            pop cx
            dec cl
        while nz | wend
        dec ch
    while nz | wend
ret
MakePatternColor:
;   di: destination
;   si: source
;   ch: count
    do
        mov cl,CharPatternSize
        do
            mov al,[si] | inc si
            mov [di],al | inc di
            dec cl
        while nz | wend
        dec ch
    while nz | wend
ret


; void ClearScreen();
cseg
Clear:
    xor ax,ax 
    do
        mov [bx],ax
        inc bx | inc bx
    wloop
ret
ClearScreen_: public ClearScreen_
    push ax | push bx | push cx
        mov ax,VramSegment | mov es,ax
        xor ax,ax
        mov bx,0
        do
            mov [es:bx],ax
            inc bx | inc bx
            cmp bx,4000h
        while nz | wend
        
		mov bx,Backup
		mov cx,VVramWidth*VVramHeight/2
        call Clear
    pop cx | pop bx | pop ax
ret


; word Put(word vram, byte c);
cseg
Put:
    add ax,CharRam
    mov si,ax
    mov cl,CharHeight/2
    do
        mov ax,[si]
        inc si | inc si
        mov [es:di],ax
        mov ax,[si]
        inc si | inc si
        mov [es:di+2000h],ax
        add di,VramWidth
        dec cl
    while nz | wend
ret
Put_: public Put_
	push ax | push cx | push si | push di
	    mov di,ax
        mov ax,VramSegment | mov es,ax
        mov al,CharPatternSize | mul dl
        call Put
	pop di | pop si | pop cx | pop ax
	add ax,VramStep
ret


; void BeginDraw();
cseg
BeginDraw_: public BeginDraw_
    push cx | push si | push di
        mov byte ptr[SpriteChar],Char_End

        mov cx,ds | mov es,cx
        mov si,VVram_
        mov di,SpriteLayer
        mov cx,VVramWidth*VVramHeight/2
        do
            movsw
        wloop
    pop di | pop si | pop cx
ret


; void EndDraw();
cseg
EndDraw_: public EndDraw_
	push ax | push cx | push dx | push bx | push si | push di
        mov ax,VramSegment | mov es,ax
        mov bx,SpriteLayer
		mov si,Backup
		mov di,Vram+Top
		mov ch,VVramHeight ; vertical count
		do
			mov cl,VVramWidth
			do 
				mov al,[bx] | inc bx
                cmp al,Char_End
                jae VVramToVram_write
				cmp al,[si]
				if ne
                    VVramToVram_write:
					mov [si],al
                    push cx | push si
                        mov ah,CharPatternSize | mul ah
                        call Put
                    pop si | pop cx
                    sub di,VramWidth*CharHeight/2
				endif
				inc di | inc di
				inc si
				dec cl
			while nz | wend
			add di,-VVramWidth*VramStep+VramRowSize
			dec ch
		while nz | wend
	pop di | pop si | pop bx | pop dx | pop cx | pop ax
ret


; void DrawSprite(byte x, byte y, byte pattern);
cseg
SourceBX:
    mov al,[bx]
    mov ah,[SpriteChar]
    mov [bx],ah
    inc byte ptr[SpriteChar]
    mov ah,CharPatternSize
    mul ah
    add ax,CharRam
    mov bx,ax
ret
MixByte:
    mov dx,[si] | inc si | inc si
    not dl
    mov al,[bx] | inc bx
    and al,dl
    or al,dh
    mov [di],al | inc di
ret
CopyByte:
    mov al,[bx] | inc bx
    mov [di],al | inc di
ret
CopyWord:
    mov ax,[bx] | inc bx | inc bx
    mov [di],ax | inc di | inc di
ret
DrawSprite_: public DrawSprite_
    push ax | push bx | push cx | push dx | push si | push di
        mov dh,al ; x
        mov bx,dx
        shr bl,1
        shr bh,1
        mov al,VVramWidth
        mul bl
        add al,bh
        add ax,SpriteLayer
        mov bx,ax
        
        mov ah,cl ; pattern
        xor al,al
        shr ax,1
        add ax,SpritePattern
        mov si,ax

        mov al,[SpriteChar]
        mov ah,CharPatternSize
        mul ah
        add ax,CharRam
        mov di,ax

        test dl,1 ; y
        if z
            test dh,1 ; x
            if z
                mov ch,2
                do
                    mov cl,2
                    do
                        push bx | push cx
                            call SourceBX
                            mov ch,CharHeight
                            do
                                mov cl,2
                                do
                                    call MixByte
                                    dec cl
                                while nz | wend
                                add si,2*2
                                dec ch
                            while nz | wend
                            add si,2*2-2*2*2*CharHeight
                        pop cx | pop bx
                        inc bx
                        dec cl
                    while nz | wend
                    add si,2*2*2*CharHeight-2*2*2
                    add bx,VVramWidth-2
                    dec ch
                while nz | wend
            else
                mov ch,2
                do
                    push bx
                        call SourceBX
                        mov cl,CharHeight
                        do
                            call CopyByte
                            call MixByte
                            add si,2*3
                            dec cl
                        while nz | wend
                        add si,2-2*4*CharHeight
                    pop bx
                    inc bx
                    push bx
                        call SourceBX
                        mov ah,CharHeight
                        do
                            mov cl,2
                            do
                                call MixByte
                                dec cl
                            while nz | wend
                            add si,2*2
                            dec ah
                        while nz | wend
                        add si,2*2-2*4*CharHeight
                    pop bx
                    inc bx
                    push bx
                        call SourceBX
                        mov cl,CharHeight
                        do
                            call MixByte
                            call CopyByte
                            add si,2*3
                            dec cl
                        while nz | wend
                        add si,-2*3
                    pop bx
                    add bx,VVramWidth-2
                    dec ch
                while nz | wend
            endif
        else
            test dh,1 ; x
            if z
                mov ch,2
                do
                    push bx
                        call SourceBX
                        mov cl,CharHeight/2
                        do
                            call CopyWord
                            dec cl
                        while nz | wend
                        mov cl,CharHeight/2
                        do
                            mov ah,2
                            do
                                call MixByte
                                dec ah
                            while nz | wend
                            add si,2*2
                            dec cl
                        while nz | wend
                    pop bx
                    inc bx
                    add si,2*2-4*2*CharHeight/2
                    dec ch
                while nz | wend
                add si,2*4*CharHeight/2-2*4
                add bx,VVramWidth-2
                mov ch,2
                do
                    push bx
                        call SourceBX
                        mov cl,CharHeight
                        do
                            mov ah,2
                            do
                                call MixByte
                                dec ah
                            while nz | wend
                            add si,2*2
                            dec cl
                        while nz | wend
                    pop bx
                    inc bx
                    add si,2*2-4*2*CharHeight
                    dec ch
                while nz | wend
                add si,2*4*CharHeight-2*4
                add bx,VVramWidth-2
                mov ch,2
                do
                    push bx
                        call SourceBX
                        mov cl,CharHeight/2
                        do
                            mov ah,2
                            do
                                call MixByte
                                dec ah
                            while nz | wend
                            add si,2*2
                            dec cl
                        while nz | wend
                        mov cl,CharHeight/2
                        do
                            call CopyWord
                            dec cl
                        while nz | wend
                    pop bx
                    inc bx
                    add si,2*2-4*2*CharHeight/2
                    dec ch
                while nz | wend
            else
                push bx
                    call SourceBX
                    mov cl,CharHeight/2
                    do
                        call CopyWord
                        dec cl
                    while nz | wend
                    mov cl,CharHeight/2
                    do
                        call CopyByte
                        call MixByte
                        add si,2*3
                        dec cl
                    while nz | wend
                pop bx
                add si,2-2*4*CharHeight/2
                inc bx
                push bx
                    call SourceBX
                    mov cl,CharHeight/2
                    do
                        call CopyWord
                        dec cl
                    while nz | wend
                    mov cl,CharHeight/2
                    do
                        mov ah,2
                        do
                            call MixByte
                            dec ah
                        while nz | wend
                        add si,2*2
                        dec cl
                    while nz | wend
                pop bx
                add si,2*2-2*4*CharHeight/2
                inc bx
                push bx
                    call SourceBX
                    mov cl,CharHeight/2
                    do
                        call CopyWord
                        dec cl
                    while nz | wend
                    mov cl,CharHeight/2
                    do
                        call MixByte
                        add si,2*3
                        call CopyByte
                        dec cl
                    while nz | wend
                pop bx
                add si,-2*3
                add bx,VVramWidth-2

                push bx
                    call SourceBX
                    mov cl,CharHeight
                    do
                        call CopyByte
                        call MixByte
                        add si,2*3
                        dec cl
                    while nz | wend
                pop bx
                add si,2-2*4*CharHeight
                inc bx
                push bx
                    call SourceBX
                    mov cl,CharHeight
                    do
                        mov ah,2
                        do
                            call MixByte
                            dec ah
                        while nz | wend
                        add si,2*2
                        dec cl
                    while nz | wend
                pop bx
                add si,2*2-2*4*CharHeight
                inc bx
                push bx
                    call SourceBX
                    mov cl,CharHeight
                    do
                        call MixByte
                        call CopyByte
                        add si,2*3
                        dec cl
                    while nz | wend
                pop bx
                add si,-2*3
                add bx,VVramWidth-2

                push bx
                    call SourceBX
                    mov cl,CharHeight/2
                    do
                        call CopyByte
                        call MixByte
                        add si,2*3
                        dec cl
                    while nz | wend
                    mov cl,CharHeight/2
                    do
                        call CopyWord
                        dec cl
                    while nz | wend
                pop bx
                add si,2-2*4*CharHeight/2
                inc bx
                push bx
                    call SourceBX
                    mov cl,CharHeight/2
                    do
                        mov ah,2
                        do
                            call MixByte
                            dec ah
                        while nz | wend
                        add si,2*2
                        dec cl
                    while nz | wend
                    mov cl,CharHeight/2
                    do
                        call CopyWord
                        dec cl
                    while nz | wend
                pop bx
                add si,2*2-2*4*CharHeight/2
                inc bx
                push bx
                    call SourceBX
                    mov cl,CharHeight/2
                    do
                        call MixByte
                        add si,2*3
                        call CopyByte
                        dec cl
                    while nz | wend
                    mov cl,CharHeight/2
                    do
                        call CopyWord
                        dec cl
                    while nz | wend
                pop bx                
            endif
        endif
    pop di | pop si | pop dx | pop cx | pop bx | pop ax
ret
